﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using FlickrScreen.Resources;
using Windows.Phone.System.UserProfile;
using FlickrScreenGenerator;
using System.Diagnostics;
using System.IO.IsolatedStorage;
using Microsoft.Phone.Scheduler;

namespace FlickrScreen
{
    public partial class MainPage : PhoneApplicationPage
    {
        private const string AgentTaskName = "Flickr";

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            if (IsolatedStorageSettings.ApplicationSettings.Contains("tag"))
                this.tag.Text = IsolatedStorageSettings.ApplicationSettings["tag"] as string;
            else
            {
                IsolatedStorageSettings.ApplicationSettings["tag"] = this.tag.Text;
                IsolatedStorageSettings.ApplicationSettings.Save();
            }

            try
            {
                if (ScheduledActionService.Find(AgentTaskName) != null)
                {
                    ScheduledActionService.Remove(AgentTaskName);
                }

                PeriodicTask task = new PeriodicTask(AgentTaskName);
                task.Description = "Change background image";
                ScheduledActionService.Add(task);

#if DEBUG
                ScheduledActionService.LaunchForTest(task.Name, TimeSpan.FromSeconds(5));
#endif
            }
            catch (InvalidOperationException exception)
            {
                if (exception.Message.Contains("BNS Error: The action is disabled"))
                {
                    MessageBox.Show("The background agent for this application has been disabled. Please go to your device settings and enable it again.");
                }
            }
            catch (SchedulerServiceException)
            {
                MessageBox.Show("Unable to add the background agent. If the number of simultaneous agents for this device has been exceeded, you can disable agents in the phone's Settings application.");
            }
        }

        private async void setApp_Click(object sender, RoutedEventArgs e)
        {
            IsolatedStorageSettings.ApplicationSettings["tag"] = this.tag.Text;
            IsolatedStorageSettings.ApplicationSettings.Save();

            // Controllo se è già impostata questa app
            if (LockScreenManager.IsProvidedByCurrentApplication)
            {
                MessageBox.Show("This app is already set to change lock screen");
            }
            else
            {
                // Chiedo il permesso
                LockScreenRequestResult r = await LockScreenManager.RequestAccessAsync();
                if (r == LockScreenRequestResult.Granted)
                {
                    MessageBox.Show("Lock screen set");
                }
                else
                {
                    MessageBox.Show("Lock screen not set");
                }
            }

            // Imposto subito una prima immagine
            if (LockScreenManager.IsProvidedByCurrentApplication)
            {
                this.progress.IsVisible = true;

                try
                {
                    // Genero la nuova immagine
                    Generator generator = new Generator();
                    var uri = await generator.GenerateLockScreenAsync();

                    // ms-appdata:///local/img.jpg
                    if (uri != null)
                    {
                        LockScreen.SetImageUri(uri);
                    }
                }
                finally
                {
                    this.progress.IsVisible = false;
                }

                Debug.WriteLine("Image set");
            }
        }


    }
}