﻿using System.Diagnostics;
using System.Windows;
using Microsoft.Phone.Scheduler;
using System.IO.IsolatedStorage;
using System;
using Windows.Phone.System.UserProfile;
using FlickrScreenGenerator;

namespace FlickrScreenAgent
{
    public class ScheduledAgent : ScheduledTaskAgent
    {
        /// <remarks>
        /// ScheduledAgent constructor, initializes the UnhandledException handler
        /// </remarks>
        static ScheduledAgent()
        {
            // Subscribe to the managed exception handler
            Deployment.Current.Dispatcher.BeginInvoke(delegate
            {
                Application.Current.UnhandledException += UnhandledException;
            });
        }

        /// Code to execute on Unhandled Exceptions
        private static void UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                Debugger.Break();
            }
        }

        /// <summary>
        /// Agent that runs a scheduled task
        /// </summary>
        /// <param name="task">
        /// The invoked task
        /// </param>
        /// <remarks>
        /// This method is called when a periodic or resource intensive task is invoked
        /// </remarks>
        protected override void OnInvoke(ScheduledTask task)
        {
            object o;
            DateTime? d = null;
            if (IsolatedStorageSettings.ApplicationSettings.TryGetValue("last", out o))
                d = (DateTime)o;
            if (d.HasValue && (DateTime.Now - d) < TimeSpan.FromHours(5))
                NotifyComplete();

            if (LockScreenManager.IsProvidedByCurrentApplication)
            {
                Deployment.Current.Dispatcher.BeginInvoke(SetImage);
            }
            else
                NotifyComplete();
        }

        private async void SetImage()
        {
            try
            {
                // Genero la nuova immagine
                Generator generator = new Generator();
                var uri = await generator.GenerateLockScreenAsync();

                // ms-appdata:///local/img.jpg
                if (uri != null)
                {
                    LockScreen.SetImageUri(uri);
                    IsolatedStorageSettings.ApplicationSettings["last"] = DateTime.Now;
                }
            }
            finally
            {
                NotifyComplete();
            }

            Debug.WriteLine("Image set");
        }
    }
}