﻿using Microsoft.Phone;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Xml.Linq;
using Windows.Graphics.Display;
using Windows.Phone.System.UserProfile;
using Windows.Storage;

namespace FlickrScreenGenerator
{
    public class Generator
    {

        public Task<Uri> GenerateLockScreenAsync()
        {
            string name = (LockScreen.GetImageUri().AbsolutePath.Contains("b")) ? "lockscreen.jpg" : "blockscreen.jpg";
            return GenerateAsync(name);
        }

        public async Task<Uri> GenerateAsync(string name)
        {
            int w = 480;
            int h = 800;
            // Definisco la dimensione dell'immagine
            switch (DisplayProperties.ResolutionScale)
            {
                case ResolutionScale.Scale160Percent:
                    w = 768;
                    h = 1280;
                    break;
                case ResolutionScale.Scale150Percent:
                    w = 720;
                    h = 1280;
                    break;
            }

            WriteableBitmap result = new WriteableBitmap(w, h);
            WriteableBitmap bitmap = null;
            // Leggo il feed e ottengo il nuovo uri
            while (bitmap == null)
            {
                Uri u = await GetNextImage();
                if (u == null) return null;

                // Carico l'immagine
                bitmap = await LoadImageAsync(u, w, h / 2).ConfigureAwait(false);
            }
            Color average = GetAverageColor(bitmap);
            result.Clear(average);
            // Disegno l'immagine in alto
            result.Blit(new Rect((w - bitmap.PixelWidth) / 2, (h / 2 - bitmap.PixelHeight) / 2, bitmap.PixelWidth, bitmap.PixelHeight), bitmap, new Rect(0, 0, bitmap.PixelWidth, bitmap.PixelHeight));

            // Salvo l'immagine finale
            var file = await ApplicationData.Current.LocalFolder
                .CreateFileAsync(name, CreationCollisionOption.ReplaceExisting)
                .AsTask()
                .ConfigureAwait(false);
            using (Stream stream = await file.OpenStreamForWriteAsync())
            {
                result.SaveJpeg(stream, w, h, 0, 100);
            }

            return new Uri("ms-appdata:///local/" + file.Name);
        }

        private async Task<Uri> GetNextImage()
        {
            WebClient c = new WebClient();
            string tag = IsolatedStorageSettings.ApplicationSettings["tag"] as string;
            if (tag == null) return null;

            tag = WebUtility.UrlEncode(tag);
            using (Stream stream = await c.OpenReadTaskAsync("http://api.flickr.com/services/feeds/photos_public.gne?tags=" + tag).ConfigureAwait(false))
            {
                XNamespace ns = "http://www.w3.org/2005/Atom";
                var list = XDocument.Load(stream)
                    .Descendants(ns + "entry")
                    .Select(e => new
                    {
                        Image = e.Elements(ns + "link").Where(l => l.Attribute("type").Value == "image/jpeg").Select(l => l.Attribute("href").Value).FirstOrDefault(),
                        ID = e.Element(ns + "id").Value
                    })
                    .Where(e => e.Image != null);

                object lastid;
                if (!IsolatedStorageSettings.ApplicationSettings.TryGetValue("lastid", out lastid))
                    lastid = "";

                var i = list.Where(l => l.ID != (string)lastid).FirstOrDefault();

                if (i != null)
                {
                    IsolatedStorageSettings.ApplicationSettings["lastid"] = i.ID;
                    IsolatedStorageSettings.ApplicationSettings.Save();
                    return new Uri(i.Image);
                }

                return null;
            }
        }

        //private static Color GetAverageColor(WriteableBitmap bitmap)
        //{
        //    Dictionary<Color, int> count = new Dictionary<Color, int>();

        //    double r = 0, g = 0, b = 0;
        //    for (int x = 0; x < bitmap.PixelWidth; x++)
        //        for (int y = 0; y < bitmap.PixelHeight; y++)
        //        {
        //            Color c = bitmap.GetPixel(x, y);
        //            c = Color.FromArgb(255, Round(c.R), Round(c.G), Round(c.B));
        //            int n;
        //            if (count.TryGetValue(c, out n))
        //            {
        //                n++;
        //                count[c] = n;
        //            }
        //            else
        //            {
        //                count[c] = 1;
        //            }
        //        }
        //    return count.Where(c => !c.Key.Equals(Colors.Black)).OrderByDescending(c => c.Value).First().Key;
        //}

        private static Color GetAverageColor(WriteableBitmap bitmap)
        {
            Dictionary<Color, int> count = new Dictionary<Color, int>();

            double r = 0, g = 0, b = 0;
            for (int x = 0; x < bitmap.PixelWidth; x++)
                for (int y = 0; y < bitmap.PixelHeight; y++)
                {
                    Color c = bitmap.GetPixel(x, y);
                    r += c.R;
                    g += c.G;
                    b += c.B;
                }

            double l = bitmap.Pixels.Length;
            r = r / l;
            g = g / l;
            b = b / l;

            return Color.FromArgb(255, (byte)r, (byte)g, (byte)b);
        }

        private static byte Round(byte color)
        {
            double c = (double)color;
            double d = c % 20;
            if (d < 10)
                return (byte)(c - d);
            else
                return (byte)(c + d);
        }

        private async Task<WriteableBitmap> LoadImageAsync(Uri u, int w, int h)
        {
            WebClient client = new WebClient();
            client.AllowReadStreamBuffering = true;

            using (Stream stream = await client.OpenReadTaskAsync(u).ConfigureAwait(false))
            {
                try
                {
                    return await Deployment.Current.Dispatcher.InvokeAsync(() => PictureDecoder.DecodeJpeg(stream, w, h));
                }
                catch
                {
                    return null;
                }
            }
        }
    }
}
